var db = {
	connection: false,
	init: function(callback) {
		try {
			if (window.openDatabase) {
				db.connection = openDatabase("TurtlePad", "1.0", "Stored Turtle Programs", 500000); //500 kB
			} else {
				alert("This Browser doesn't support local storage.");
				return;
			}
		} catch(err) {
			alert("Problem while opening local database: "+err);
			return;
		}
		if (!db.connection) alert("Unable to open local database.");
		db.connection.transaction(function(tx) {
			tx.executeSql(
				"CREATE TABLE IF NOT EXISTS Programs (id INTEGER PRIMARY KEY ASC, name TEXT, code TEXT, created REAL, updated REAL)",
				[],
				function(result) {
					if(callback) callback();
				},
				function(tx, error) {
					alert('could not create db '+error.message);
				}
			);
		});

	},
	addProg: function(callback) {
		var name = 'Untitled Program';
		var code = '; Untitled Turtle App\nrepeat 5 [fd 100 rt 144]\nPENUP\nfd 200\nPENDOWN\nrepeat 5 [fd 100 rt 144] \nPENUP\nrt 300\nfd 200\nPENDOWN\nFORWARD 100\nLEFT 90\nFORWARD 100\nLEFT 90\nFORWARD 100\nLEFT 90\nFORWARD 100\nLEFT 90';
		var created = new Date().getTime();;
		var updated = new Date().getTime();;
		db.connection.transaction(function(tx) {
			tx.executeSql(
				"INSERT INTO Programs (name, code, created, updated) VALUES (?, ?, ?, ?)",
				[name, code, created, updated],
				function(result) {
					if(callback) callback();
				},
				function(tx, error) {
					alert('could not insert: '+error.message);
				}
			);
		});
	},
	deleteProg: function(id, callback) {
		db.connection.transaction(function(tx) {
			tx.executeSql(
				"DELETE FROM Programs WHERE id = ?",
				[id],
				function(result) {
					if(callback) callback();
				},
				function(tx, error) {
					alert('could not delete: '+error.message);
				}
			);
		});
	},
	updateProg: function(id, name, code, callback) {
		var updated = new Date().getTime();;
		db.connection.transaction(function(tx) {
			tx.executeSql(
				"UPDATE Programs SET  name=?, code=?, updated=? WHERE id=?",
				[name, code, updated, id],
				function(result) {
					if(callback) callback();
				},
				function(tx, error) {
					alert('could not insert: '+error.message);
				}
			);
		});
	},
	getProgs: function(callback) {
		db.connection.transaction(function(tx) {
			tx.executeSql(
				"SELECT id, name, code, created, updated FROM Programs",
				[],
				function(tx, result) {
					var progs = [];
					for(var i=0; i<result.rows.length; i++) {
						progs.push(result.rows.item(i));
					}
					if(callback) callback(progs);
				},
				function(tx, error) {
					alert('Failed to retrieve Programs from database: ' + error.message);
				}
			);
		});
	},
	getProg: function(id, callback) {
		db.connection.transaction(function(tx) {
			tx.executeSql(
				"SELECT id, name, code, created, updated FROM Programs WHERE id=?",
				[id],
				function(tx, result) {
					var prog = result.rows.item(0);
					if(callback) callback(prog);
				},
				function(tx, error) {
					alert('Failed to retrieve Program from database: ' + error.message);
				}
			);
		});
	}
}