var layout = {
	redrawProgramList: function(id) {
		if(id) {
			var progs = db.getProg(id, layout.redrawSingeProgramCallback);
		} else {
			$('#programlist').empty();
			var progs = db.getProgs(layout.redrawProgramListCallback);
		}
	},
	redrawProgramListCallback: function(progs) {
		for(var i=0; i<progs.length; i++) {
			var div = document.createElement('div');
			$(div).addClass('sidebarbutton');
			$(div).attr('programid',progs[i]['id']);
			$(div).attr('id','programbutton'+progs[i]['id']);
			$(div).click(programs.open);
			
			var text = document.createElement('div');
			$(text).attr('id','text'+progs[i]['id']);
			var code = progs[i]['code'];
			layout.addProgramListText(text, code);
			$(div).append(text);
			
			$('#programlist').append(div);
			
			layout.addProgramListImg(div, progs[i]['code']);
		}
	},
	redrawSingeProgramCallback: function(prog) {
		$('#programbutton'+prog['id']+' img').remove();
		layout.addProgramListImg($('#programbutton'+prog['id']), prog['code']);
		
		$('#text'+prog['id']).empty();
		layout.addProgramListText($('#text'+prog['id']), prog['code']);
	},
	addProgramListImg: function(div, code) {
		programs.drawTurtle(code, function(warnings, imagename, width, height) {
			var img = document.createElement('img');
			$(img).attr('src','/tmp/'+imagename);
			$(div).append(img);
		}, 60, 50);
	},
	addProgramListText: function(text, code) {
		var lines = code.split('\n');
		for(var j=0; j<lines.length && j<3; j++) {
			var line = document.createElement('p');
			if(j == 0) {
				var firstline = document.createElement('b');
				$(line).append(firstline);
				$(firstline).text(lines[0]);
			} else {
				$(line).text(lines[j]);
			}
			$(text).append(line);
		}
	},
	code_has_focus: false,
	codeFocus: function() {
		layout.code_has_focus = true;
		layout.resizeElements();
	},
	codeBlur: function() {
		layout.code_has_focus = false;
		layout.resizeElements();
	},
	resizeElements: function() {
		var w = $(window);
		var p = $('#program');
		var height = w.height();
		
		if(layout.code_has_focus) {
			// try to figure out whether there is an on-screen keyboard,
			// and if so, how big it is
			var keyboardheight = 0;
			if(navigator.userAgent.match(/iPad/i)) {
				if(w.width() > w.height()) {
					keyboardheight = 396; // landscape
				} else {
					keyboardheight = 307; // portrait
				}
			} else if((navigator.userAgent.match(/iPhone/i)) || (navigator.userAgent.match(/iPod/i))) {
				if(w.width() > w.height()) {
					keyboardheight = 70; // landscape
				} else {
					keyboardheight = 70; // portrait
				}
			}
			height -= keyboardheight;
			height = Math.max(60,height);
		}
		$('#content').css({
			height:height,
			width:w.width()
		});
		
		$('#sidebar').css({
			height:height
		});
		$('#programlist').css({
			height:Math.max(20,height-70)
		});
		p.css({
			width:Math.max(20,w.width()-70),
			height:Math.max(20,height-20)
		});
		
		$('#codearea').css({
			height:Math.max(20,p.height()-50),
			width:Math.max(20,p.width()-260)
		});
		$('#informationarea').css({
			height:Math.max(20,p.height()-50)
		});
		
		layout.resetScroll();
	},
	displayMainTurtleImage: function(warnings, imagename, width, height) {
		$('#informationarea').empty();
		
		// show the turtle image
		var img = document.createElement('img');
		$(img).addClass('mainturtleimage');
		$(img).attr('src','/tmp/'+imagename);
		$(img).attr('imagename',imagename);
		$(img).attr('width',width);
		$(img).attr('height',height);
		$(img).css({
			width:Math.max(20,$('#informationarea').width()-10),
			height:height * ($('#informationarea').width()-10) / width
		});
		$(img).click(layout.showLargeTurtleImage);
		$('#informationarea').append(img);
		
		// show warnings, if any
		if(warnings) {
			var warnings = document.createElement('div');
			$(warnings).addClass('warnings');
			
			var title = document.createElement('div');
			$(title).addClass('warningstitle');
			$(title).text('Warnings');
			
			var text = document.createElement('div');
			$(text).text(warnings);
			
			$(warnings).append(title);
			$(warnings).append(text);
			$('#informationarea').append(warnings);
		}
	},
	showLargeTurtleImage: function() {
		var img = document.createElement('img');
		$(img).addClass('largeturtleimage');
		$(img).attr('src','/tmp/'+$(this).attr('imagename'));
		var height = $(this).attr('height');
		var width = $(this).attr('width');
		
		var zoomfactor = $(window).width()/width;
		if(zoomfactor > ($(window).height()/height)) zoomfactor = $(window).height()/height;
		
		$(img).css({
			width:zoomfactor * width,
			height:zoomfactor * height,
			left:parseInt(($(window).width() - (zoomfactor * width))/2),
			top:parseInt(($(window).height() - (zoomfactor * height))/2)
		});
		$(img).click(function(event) {
			event.stopPropagation();
			$('.largeturtleimage').remove();
		});
		$(document.body).append(img);
	},
	resetScroll: function() {
		var w = $(window);
		w.scrollTop(0);
		w.scrollLeft(0);
	},
	program_visible: false,
	showProgram: function() {
		if(layout.program_visible) return;
		layout.program_visible = true;
		
		$('#infoelements').animate({
			opacity:0
		},500, function() {
			$('#infoelements').css({
				display:'none'
			})
		});
		$('#programelements').css({
			display:'block',
			opacity:0
		})
		$('#programelements').animate({
			opacity:1.0
		});
		
		$('#program').animate({
			right:10
		},500);
		$('#sidebar').animate({
			opacity:0.5
		},500);
	},
	hideProgram: function() {
		if(!layout.program_visible) return;
		
		if(programs.opened_program)
			layout.redrawProgramList(programs.opened_program['id']);
		layout.program_visible = false;
		
		$('#programelements').animate({
			opacity:0
		},500, function() {
			$('#programelements').css({
				display:'none'
			})
		});
		$('#infoelements').css({
			display:'block',
			opacity:0
		})
		$('#infoelements').animate({
			opacity:1.0
		});
		
		$('#program').animate({
			right:-210
		},500);
		$('#sidebar').animate({
			opacity:1
		},500);
		
		$('.activesidebarbutton').removeClass('activesidebarbutton');
	}
}